/* (Auto-generated binary data file). */

#ifndef BINARY_DISTRHOARTWORKAMPLITUDEIMPOSER_HPP
#define BINARY_DISTRHOARTWORKAMPLITUDEIMPOSER_HPP

namespace DistrhoArtworkAmplitudeImposer
{
    extern const char* backData;
    const unsigned int backDataSize = 90000;
    const unsigned int backWidth    = 300;
    const unsigned int backHeight   = 150;

    extern const char* sliderData;
    const unsigned int sliderDataSize = 512;
    const unsigned int sliderWidth    = 16;
    const unsigned int sliderHeight   = 16;
}

#endif // BINARY_DISTRHOARTWORKAMPLITUDEIMPOSER_HPP

