/* (Auto-generated binary data file). */

#ifndef BINARY_DISTRHOARTWORK3BANDEQ_HPP
#define BINARY_DISTRHOARTWORK3BANDEQ_HPP

namespace DistrhoArtwork3BandEQ
{
    extern const char* aboutData;
    const unsigned int aboutDataSize = 172710;
    const unsigned int aboutWidth    = 303;
    const unsigned int aboutHeight   = 190;

    extern const char* aboutButtonHoverData;
    const unsigned int aboutButtonHoverDataSize = 5888;
    const unsigned int aboutButtonHoverWidth    = 92;
    const unsigned int aboutButtonHoverHeight   = 16;

    extern const char* aboutButtonNormalData;
    const unsigned int aboutButtonNormalDataSize = 5888;
    const unsigned int aboutButtonNormalWidth    = 92;
    const unsigned int aboutButtonNormalHeight   = 16;

    extern const char* backgroundData;
    const unsigned int backgroundDataSize = 437472;
    const unsigned int backgroundWidth    = 392;
    const unsigned int backgroundHeight   = 372;

    extern const char* knobData;
    const unsigned int knobDataSize = 15376;
    const unsigned int knobWidth    = 62;
    const unsigned int knobHeight   = 62;

    extern const char* sliderData;
    const unsigned int sliderDataSize = 6000;
    const unsigned int sliderWidth    = 50;
    const unsigned int sliderHeight   = 30;
}

#endif // BINARY_DISTRHOARTWORK3BANDEQ_HPP

