/* (Auto-generated binary data file). */

#ifndef BINARY_DISTRHOARTWORKSOULFORCE_HPP
#define BINARY_DISTRHOARTWORKSOULFORCE_HPP

namespace DistrhoArtworkSoulForce
{
    extern const char* aboutData;
    const unsigned int aboutDataSize = 378080;
    const unsigned int aboutWidth    = 278;
    const unsigned int aboutHeight   = 340;

    extern const char* backgroundData;
    const unsigned int backgroundDataSize = 283560;
    const unsigned int backgroundWidth    = 278;
    const unsigned int backgroundHeight   = 340;

    extern const char* button_downData;
    const unsigned int button_downDataSize = 4959;
    const unsigned int button_downWidth    = 29;
    const unsigned int button_downHeight   = 57;

    extern const char* button_upData;
    const unsigned int button_upDataSize = 4959;
    const unsigned int button_upWidth    = 29;
    const unsigned int button_upHeight   = 57;

    extern const char* knob_feedbackData;
    const unsigned int knob_feedbackDataSize = 2097600;
    const unsigned int knob_feedbackWidth    = 80;
    const unsigned int knob_feedbackHeight   = 8740;

    extern const char* knob_shapeData;
    const unsigned int knob_shapeDataSize = 2097600;
    const unsigned int knob_shapeWidth    = 80;
    const unsigned int knob_shapeHeight   = 8740;

    extern const char* led_offData;
    const unsigned int led_offDataSize = 3570;
    const unsigned int led_offWidth    = 34;
    const unsigned int led_offHeight   = 35;

    extern const char* led_onData;
    const unsigned int led_onDataSize = 3570;
    const unsigned int led_onWidth    = 34;
    const unsigned int led_onHeight   = 35;

    extern const char* switch_leftData;
    const unsigned int switch_leftDataSize = 6273;
    const unsigned int switch_leftWidth    = 51;
    const unsigned int switch_leftHeight   = 41;

    extern const char* switch_rightData;
    const unsigned int switch_rightDataSize = 6273;
    const unsigned int switch_rightWidth    = 51;
    const unsigned int switch_rightHeight   = 41;
}

#endif // BINARY_DISTRHOARTWORKSOULFORCE_HPP

