/* (Auto-generated binary data file). */

#ifndef BINARY_DISTRHOARTWORKMVERB_HPP
#define BINARY_DISTRHOARTWORKMVERB_HPP

namespace DistrhoArtworkMVerb
{
    extern const char* backgroundData;
    const unsigned int backgroundDataSize = 147744;
    const unsigned int backgroundWidth    = 456;
    const unsigned int backgroundHeight   = 108;

    extern const char* knobData;
    const unsigned int knobDataSize = 528384;
    const unsigned int knobWidth    = 32;
    const unsigned int knobHeight   = 4128;
}

#endif // BINARY_DISTRHOARTWORKMVERB_HPP

