/* (Auto-generated binary data file). */

#ifndef BINARY_DISTRHOARTWORKPINGPONGPAN_HPP
#define BINARY_DISTRHOARTWORKPINGPONGPAN_HPP

namespace DistrhoArtworkPingPongPan
{
    extern const char* aboutData;
    const unsigned int aboutDataSize = 172710;
    const unsigned int aboutWidth    = 303;
    const unsigned int aboutHeight   = 190;

    extern const char* aboutButtonHoverData;
    const unsigned int aboutButtonHoverDataSize = 7600;
    const unsigned int aboutButtonHoverWidth    = 95;
    const unsigned int aboutButtonHoverHeight   = 20;

    extern const char* aboutButtonNormalData;
    const unsigned int aboutButtonNormalDataSize = 7600;
    const unsigned int aboutButtonNormalWidth    = 95;
    const unsigned int aboutButtonNormalHeight   = 20;

    extern const char* backgroundData;
    const unsigned int backgroundDataSize = 157080;
    const unsigned int backgroundWidth    = 308;
    const unsigned int backgroundHeight   = 170;

    extern const char* knobData;
    const unsigned int knobDataSize = 17956;
    const unsigned int knobWidth    = 67;
    const unsigned int knobHeight   = 67;
}

#endif // BINARY_DISTRHOARTWORKPINGPONGPAN_HPP

