//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by dbssaver.rc
//
#define IDS_APPNAME                     2
#define DLG_CONFIG                      104
#define IDB_BITMAP1                     108
#define IDC_FLASH                       1000

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        114
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1005
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
