/**
 * Dummy file for Xcode, as it doesn't create static libs with only $<TARGET_OBJECTS:...> sources.
 * Linking this file will not add any code to the executables as the function is never referenced.
 */

void DummyFunction() {}
