/* (Auto-generated binary data file). */

#ifndef BINARY_DISTRHOARTWORKNEKOBI_HPP
#define BINARY_DISTRHOARTWORKNEKOBI_HPP

namespace DistrhoArtworkNekobi
{
    extern const char* aboutData;
    const unsigned int aboutDataSize = 172710;
    const unsigned int aboutWidth    = 303;
    const unsigned int aboutHeight   = 190;

    extern const char* aboutButtonHoverData;
    const unsigned int aboutButtonHoverDataSize = 5888;
    const unsigned int aboutButtonHoverWidth    = 92;
    const unsigned int aboutButtonHoverHeight   = 16;

    extern const char* aboutButtonNormalData;
    const unsigned int aboutButtonNormalDataSize = 5888;
    const unsigned int aboutButtonNormalWidth    = 92;
    const unsigned int aboutButtonNormalHeight   = 16;

    extern const char* backgroundData;
    const unsigned int backgroundDataSize = 206064;
    const unsigned int backgroundWidth    = 636;
    const unsigned int backgroundHeight   = 108;

    extern const char* claw1Data;
    const unsigned int claw1DataSize = 4096;
    const unsigned int claw1Width    = 32;
    const unsigned int claw1Height   = 32;

    extern const char* claw2Data;
    const unsigned int claw2DataSize = 4096;
    const unsigned int claw2Width    = 32;
    const unsigned int claw2Height   = 32;

    extern const char* knobData;
    const unsigned int knobDataSize = 10000;
    const unsigned int knobWidth    = 50;
    const unsigned int knobHeight   = 50;

    extern const char* run1Data;
    const unsigned int run1DataSize = 4096;
    const unsigned int run1Width    = 32;
    const unsigned int run1Height   = 32;

    extern const char* run2Data;
    const unsigned int run2DataSize = 4096;
    const unsigned int run2Width    = 32;
    const unsigned int run2Height   = 32;

    extern const char* run3Data;
    const unsigned int run3DataSize = 4096;
    const unsigned int run3Width    = 32;
    const unsigned int run3Height   = 32;

    extern const char* run4Data;
    const unsigned int run4DataSize = 4096;
    const unsigned int run4Width    = 32;
    const unsigned int run4Height   = 32;

    extern const char* scratch1Data;
    const unsigned int scratch1DataSize = 4096;
    const unsigned int scratch1Width    = 32;
    const unsigned int scratch1Height   = 32;

    extern const char* scratch2Data;
    const unsigned int scratch2DataSize = 4096;
    const unsigned int scratch2Width    = 32;
    const unsigned int scratch2Height   = 32;

    extern const char* sitData;
    const unsigned int sitDataSize = 4096;
    const unsigned int sitWidth    = 32;
    const unsigned int sitHeight   = 32;

    extern const char* sliderData;
    const unsigned int sliderDataSize = 6084;
    const unsigned int sliderWidth    = 39;
    const unsigned int sliderHeight   = 39;

    extern const char* tailData;
    const unsigned int tailDataSize = 4096;
    const unsigned int tailWidth    = 32;
    const unsigned int tailHeight   = 32;
}

#endif // BINARY_DISTRHOARTWORKNEKOBI_HPP

