//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by VizKit.rc
//
#define IDR_DEVROYEFONT                 134
#define IDD_DIALOG1                     1000
#define IDC_COMBO1                      1001
#define IDD_DIALOG2                     1002
#define IDC_BUTTON1                     1002
#define IDD_DIALOG3                     1003
#define IDC_RICHEDIT1                   1003
#define IDC_CHECK1                      1004
#define IDC_CHECK_FOR_UPDATE            1004
#define IDR_PNG_SPOT                    1005
#define IDD_DIALOG4                     1006

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        1007
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1005
#define _APS_NEXT_SYMED_VALUE           1000
#endif
#endif
