//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by projectM-wmp.rc
//
#define IDS_PROJNAME                    100
#define IDS_EFFECTNAME                  101
#define IDS_BARSPRESETNAME              102
#define IDS_SCOPEPRESETNAME             103
#define IDR_PROJECTMWMP                 104
#define IDS_DESCRIPTION                 105

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        201
#define _APS_NEXT_COMMAND_VALUE         32768
#define _APS_NEXT_CONTROL_VALUE         201
#define _APS_NEXT_SYMED_VALUE           105
#endif
#endif
