//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by plugin.rc
//
#define IDS_ABOUT_STRING                1
#define IDS_SZ_MENU_NAV_TOOLTIP         2
#define ID_DOCS                         3
#define IDS_UNTITLED_MENU_ITEM          3
#define IDLEFT                          4
#define IDS_UNTITLED_MENU               4
#define ID_WEB                          5
#define IDS_ON                          5
#define IDRIGHT                         6
#define IDS_OFF                         6
#define ID_DEFAULTS                     7
#define IDS_USE_UP_DOWN_ARROW_KEYS      7
#define ID_MSG                          8
#define IDS_CURRENT_VALUE_OF_X          8
#define ID_SPRITE                       9
#define IDS_LOAD_FROM_FILE              9
#define IDS_SAVE_TO_FILE                10
#define IDS_ENTER_THE_NEW_STRING        11
#define IDS_MILKDROP_ERROR              14
#define IDS_MILKDROP_WARNING            19
#define IDS_ERROR_CREATING_DOUBLE_SIZED_GDI_TITLE_FONT 20
#define IDS_ERROR_CREATING_DOUBLE_SIZED_D3DX_TITLE_FONT 21
#define IDS_RATING                      23
#define IDS_DISABLED                    24
#define IDS_ARE_YOU_SURE_YOU_WANT_TO_DELETE_PRESET 26
#define IDS_PRESET_TO_DELETE            27
#define IDS_FILE_ALREADY_EXISTS_OVERWRITE_IT 28
#define IDS_FILE_IN_QUESTION_X_MILK     29
#define IDS_ERROR_NO_PRESET_FILE_FOUND_IN_X_MILK 30
#define IDS_LOAD_WHICH_PRESET_PLUS_COMMANDS 31
#define IDS_PAGE_X_OF_X                 32
#define IDS_LOCKED                      33
#define IDS_ILLEGAL_CHARACTER           37
#define IDS_STRING_TOO_LONG             38
#define IDS_DIRECTORY_TO_JUMP_TO        39
#define IDS_ERROR_IMPORTING_BAD_FILENAME 40
#define IDS_ERROR_IMPORTING_BAD_FILENAME_OR_NOT_OVERWRITEABLE 41
#define IDS_INVALID_PATH                42
#define IDS_ENTER_THE_NEW_NAME_FOR_X    43
#define IDS_PRESET_ORDER_IS_NOW_X       44
#define IDS_SEQUENTIAL                  45
#define IDS_RANDOM                      46
#define IDS_SAVE_AS                     50
#define IDS_AUTO                        51
#define IDS_8X6_FAST                    52
#define IDS_16X12_FAST                  53
#define IDS_24X18                       54
#define IDS_32X24                       55
#define IDS_40X30                       56
#define IDS_48X36_DEFAULT               57
#define IDS_64X48_SLOW                  58
#define IDS_80X60_SLOW                  59
#define IDS_96X72_SLOW                  60
#define IDS_128X96_SLOW                 61
#define IDS_ERROR_IN_SHELLEXECUTE       62
#define IDS_MESH_SIZE                   65
#define IDS_MESH_SIZE_TEXT              66
#define IDS_CB_ALWAYS3D                 67
#define IDS_DISABLE_PRESET_RATING       70
#define IDS_DISABLE_PRESET_RATING_TEXT  71
#define IDS_CB_NOWARN2                  73
#define IDS_START_WITH_PRESET_LOCK_ON   76
#define IDS_START_WITH_PRESET_LOCK_ON_TEXT 77
#define IDS_BRIGHT_SLIDER               78
#define IDS_CB_AUTOGAMMA                79
#define IDS_SPRITE                      80
#define IDS_MSG                         81
#define IDS_SONGTITLEANIM_DURATION_TEXT 86
#define IDS_RAND_TITLE_TEXT             87
#define IDS_RAND_MSG_TEXT               88
#define IDS_TITLE_ANIMS_TEXT            89
#define IDS_BETWEEN_TIME_TEXT           90
#define IDS_BETWEEN_TIME_RANDOM_TEXT    91
#define IDS_BLEND_AUTO_TEXT             92
#define IDS_BLEND_USER_TEXT             93
#define IDS_HARDCUT_BETWEEN_TIME_TEXT   94
#define IDS_HARDCUT_LOUDNESS_TEXT       95
#define IDS_CB_HARDCUTS                 96
#define IDS_EDIT_CURRENT_PRESET         98
#define IDS_MOTION                      99
#define IDS_DRAWING_CUSTOM_SHAPES       100
#define IDD_DIALOG1                     101
#define IDD_CONFIG                      101
#define IDS_DRAWING_CUSTOM_WAVES        101
#define IDD_PROPPAGE_1                  102
#define IDS_DRAWING_SIMPLE_WAVEFORM     102
#define IDD_PROPPAGE_2                  103
#define IDS_DRAWING_BORDERS_MOTION_VECTORS 103
#define IDS_POST_PROCESSING_MISC        104
#define IDS_CUSTOM_WAVE_X               105
#define IDS_CUSTOM_SHAPE_X              106
#define IDS_MENU_EDIT_PRESET_INIT_CODE  107
#define IDS_MENU_EDIT_PRESET_INIT_CODE_TT 108
#define IDS_MENU_EDIT_PER_FRAME_EQUATIONS 109
#define IDR_WINDOWED_CONTEXT_MENU       110
#define IDS_MENU_EDIT_PER_FRAME_EQUATIONS_TT 110
#define IDS_MENU_WAVE_TYPE              113
#define IDD_PROPPAGE_3                  114
#define IDS_MENU_WAVE_TYPE_TT           114
#define IDD_PROPPAGE_4                  115
#define IDS_MENU_SIZE                   115
#define IDD_PROPPAGE_5                  116
#define IDS_MENU_SIZE_TT                116
#define IDD_PROPPAGE_6                  117
#define IDS_MENU_SMOOTH                 117
#define IDD_PROPPAGE_7                  118
#define IDS_MENU_SMOOTH_TT              118
#define IDD_PROPPAGE_8                  119
#define IDS_MENU_MYSTERY_PARAMETER      119
#define IDI_PLUGIN_ICON                 120
#define IDS_MENU_MYSTERY_PARAMETER_TT   120
#define IDD_FONTDIALOG                  121
#define IDS_MENU_POSITION_X             121
#define IDD_DESKTOPMODE                 122
#define IDS_MENU_POSITION_X_TT          122
#define IDD_DUALHEAD                    123
#define IDS_MENU_POSITION_Y             123
#define IDS_MENU_POSITION_Y_TT          124
#define IDS_MENU_COLOR_RED              125
#define IDS_MENU_COLOR_RED_TT           126
#define IDS_MENU_COLOR_GREEN            127
#define IDS_MENU_COLOR_GREEN_TT         128
#define IDR_TEXT1                       129
#define IDS_MENU_COLOR_BLUE             129
#define IDR_TEXT2                       130
#define IDS_MENU_COLOR_BLUE_TT          130
#define IDS_MENU_OPACITY                131
#define IDS_MENU_OPACITY_TT             132
#define IDS_MENU_USE_DOTS               133
#define IDS_MENU_USE_DOTS_TT            134
#define IDS_MENU_DRAW_THICK             135
#define IDS_MENU_DRAW_THICK_TT          136
#define IDS_MENU_MODULATE_OPACITY_BY_VOLUME 137
#define IDS_MENU_MODULATE_OPACITY_BY_VOLUME_TT 138
#define IDS_MENU_MODULATION_TRANSPARENT_VOLUME 139
#define IDS_MENU_MODULATION_TRANSPARENT_VOLUME_TT 140
#define IDS_MENU_MODULATION_OPAQUE_VOLUME 141
#define IDS_MENU_MODULATION_OPAQUE_VOLUME_TT 142
#define IDS_MENU_ADDITIVE_DRAWING       143
#define IDS_MENU_ADDITIVE_DRAWING_TT    144
#define IDS_MENU_COLOR_BRIGHTENING      145
#define IDS_MENU_COLOR_BRIGHTENING_TT   146
#define IDS_MENU_OUTER_BORDER_THICKNESS 147
#define IDS_MENU_OUTER_BORDER_THICKNESS_TT 148
#define IDS_MENU_COLOR_RED_OUTER        149
#define IDS_MENU_COLOR_RED_OUTER_TT     150
#define IDS_MENU_COLOR_GREEN_OUTER      151
#define IDS_MENU_COLOR_GREEN_OUTER_TT   152
#define IDS_MENU_COLOR_BLUE_OUTER       153
#define IDS_MENU_COLOR_BLUE_OUTER_TT    154
#define IDS_MENU_OPACITY_OUTER          155
#define IDS_MENU_OPACITY_OUTER_TT       156
#define IDS_MENU_INNER_BORDER_THICKNESS 157
#define IDS_MENU_INNER_BORDER_THICKNESS_TT 160
#define IDS_MENU_COLOR_RED_INNER_TT     161
#define IDS_MENU_COLOR_GREEN_INNER_TT   162
#define IDS_MENU_COLOR_BLUE_INNER_TT    163
#define IDS_MENU_OPACITY_INNER_TT       164
#define IDS_MENU_MOTION_VECTOR_OPACITY  165
#define IDS_MENU_MOTION_VECTOR_OPACITY_TT 167
#define IDS_MENU_NUM_MOT_VECTORS_X      168
#define IDS_MENU_NUM_MOT_VECTORS_X_TT   169
#define IDS_MENU_NUM_MOT_VECTORS_Y      170
#define IDS_MENU_NUM_MOT_VECTORS_Y_TT   171
#define IDS_MENU_OFFSET_X               172
#define IDS_MENU_OFFSET_X_TT            173
#define IDS_MENU_OFFSET_Y               174
#define IDS_MENU_OFFSET_Y_TT            175
#define IDS_MENU_TRAIL_LENGTH           176
#define IDS_MENU_TRAIL_LENGTH_TT        177
#define IDS_MENU_COLOR_RED_MOTION_VECTOR_TT 178
#define IDS_MENU_COLOR_GREEN_MOTION_VECTOR_TT 179
#define IDS_MENU_COLOR_BLUE_MOTION_VECTOR_TT 180
#define IDS_MENU_ZOOM_AMOUNT            181
#define IDS_MENU_ZOOM_AMOUNT_TT         182
#define IDS_MENU_ZOOM_EXPONENT          183
#define IDS_MENU_ZOOM_EXPONENT_TT       184
#define IDS_MENU_WARP_AMOUNT            185
#define IDS_MENU_WARP_AMOUNT_TT         186
#define IDS_MENU_WARP_SCALE             187
#define IDS_MENU_WARP_SCALE_TT          188
#define IDS_MENU_WARP_SPEED             189
#define IDS_MENU_WARP_SPEED_TT          190
#define IDS_MENU_ROTATION_AMOUNT        191
#define IDS_MENU_ROTATION_AMOUNT_TT     192
#define IDS_MENU_ROTATION_CENTER_OF_X   193
#define IDS_MENU_ROTATION_CENTER_OF_X_TT 194
#define IDS_MENU_ROTATION_CENTER_OF_Y   195
#define IDS_MENU_ROTATION_CENTER_OF_Y_TT 196
#define IDS_MENU_TRANSLATION_X          197
#define IDS_MENU_TRANSLATION_X_TT       198
#define IDS_MENU_TRANSLATION_Y          199
#define IDS_MENU_TRANSLATION_Y_TT       200
#define IDS_MENU_SCALING_X              201
#define IDS_MENU_SCALING_X_TT           202
#define IDS_MENU_SCALING_Y              203
#define IDS_MENU_SCALING_Y_TT           204
#define IDS_MENU_SUSTAIN_LEVEL          205
#define IDS_MENU_SUSTAIN_LEVEL_TT       206
#define IDS_MENU_DARKEN_CENTER          207
#define IDS_MENU_DARKEN_CENTER_TT       208
#define IDS_MENU_GAMMA_ADJUSTMENT       209
#define IDS_MENU_GAMMA_ADJUSTMENT_TT    210
#define IDS_MENU_HUE_SHADER             211
#define IDS_MENU_HUE_SHADER_TT          212
#define IDS_MENU_VIDEO_ECHO_ALPHA       213
#define IDS_MENU_VIDEO_ECHO_ALPHA_TT    214
#define IDS_MENU_VIDEO_ECHO_ZOOM        215
#define IDS_MENU_VIDEO_ECHO_ZOOM_TT     216
#define IDS_MENU_VIDEO_ECHO_ORIENTATION 217
#define IDS_MENU_VIDEO_ECHO_ORIENTATION_TT 218
#define IDS_MENU_TEXTURE_WRAP           219
#define IDS_MENU_TEXTURE_WRAP_TT        220
#define IDS_MENU_FILTER_INVERT          223
#define IDS_MENU_FILTER_INVERT_TT       224
#define IDS_MENU_FILTER_BRIGHTEN        225
#define IDS_MENU_FILTER_BRIGHTEN_TT     226
#define IDS_MENU_FILTER_DARKEN          227
#define IDS_MENU_FILTER_DARKEN_TT       228
#define IDS_MENU_FILTER_SOLARIZE        229
#define IDS_MENU_FILTER_SOLARIZE_TT     230
#define IDS_MENU_ENABLED                231
#define IDS_MENU_ENABLED_TT             232
#define IDS_MENU_NUMBER_OF_SAMPLES      233
#define IDS_MENU_NUMBER_OF_SAMPLES_TT   234
#define IDS_MENU_L_R_SEPARATION         235
#define IDS_MENU_L_R_SEPARATION_TT      236
#define IDS_MENU_SCALING                237
#define IDS_MENU_SCALING_TT             238
#define IDS_MENU_SMOOTHING_TT           239
#define IDS_MENU_OPACITY_WAVE_TT        240
#define IDS_MENU_USE_SPECTRUM           241
#define IDS_MENU_USE_SPECTRUM_TT        242
#define IDS_MENU_USE_DOTS_WAVE_TT       243
#define IDS_MENU_DRAW_THICK_WAVE_TT     244
#define IDS_MENU_ADDITIVE_DRAWING_WAVE_TT 245
#define IDS_MENU_EXPORT_TO_FILE         246
#define IDS_MENU_EXPORT_TO_FILE_TT      247
#define IDS_MENU_IMPORT_FROM_FILE       248
#define IDS_MENU_IMPORT_FROM_FILE_TT    249
#define IDS_MENU_EDIT_INIT_CODE         250
#define IDS_MENU_EDIT_INIT_CODE_TT      251
#define IDS_MENU_EDIT_PER_FRAME_CODE    252
#define IDS_MENU_EDIT_PER_FRAME_CODE_TT 253
#define IDS_MENU_EDIT_PER_POINT_CODE    254
#define IDS_MENU_EDIT_PER_POINT_CODE_TT 255
#define IDS_MENU_ENABLED_SHAPE_TT       256
#define IDS_MENU_NUMBER_OF_SIDES        257
#define IDS_MENU_NUMBER_OF_SIDES_TT     258
#define IDS_MENU_DRAW_THICK_SHAPE_TT    259
#define IDS_MENU_ADDITIVE_DRAWING_SHAPE_TT 260
#define IDS_MENU_X_POSITION             261
#define IDS_MENU_X_POSITION_TT          262
#define IDS_MENU_Y_POSITION             263
#define IDS_MENU_Y_POSITION_TT          264
#define IDS_MENU_RADIUS                 265
#define IDS_MENU_RADIUS_TT              266
#define IDS_MENU_ANGLE                  267
#define IDS_MENU_ANGLE_TT               268
#define IDS_MENU_TEXTURED               269
#define IDS_MENU_TEXTURED_TT            270
#define IDS_MENU_TEXTURE_ZOOM           271
#define IDS_MENU_TEXTURE_ZOOM_TT        272
#define IDS_MENU_TEXTURE_ANGLE          273
#define IDS_MENU_TEXTURE_ANGLE_TT       274
#define IDS_MENU_INNER_COLOR_RED        275
#define IDS_MENU_INNER_COLOR_RED_TT     276
#define IDS_MENU_INNER_COLOR_GREEN      277
#define IDS_MENU_INNER_COLOR_GREEN_TT   278
#define IDS_MENU_INNER_COLOR_BLUE       279
#define IDS_MENU_INNER_COLOR_BLUE_TT    280
#define IDS_MENU_INNER_OPACITY          281
#define IDS_MENU_INNER_OPACITY_TT       282
#define IDS_MENU_OUTER_COLOR_RED        283
#define IDS_MENU_OUTER_COLOR_RED_TT     284
#define IDS_MENU_OUTER_COLOR_GREEN      285
#define IDS_MENU_OUTER_COLOR_GREEN_TT   286
#define IDS_MENU_OUTER_COLOR_BLUE       287
#define IDS_MENU_OUTER_COLOR_BLUE_TT    288
#define IDS_MENU_OUTER_OPACITY          289
#define IDS_MENU_OUTER_OPACITY_TT       290
#define IDS_MENU_BORDER_COLOR_RED       291
#define IDS_MENU_BORDER_COLOR_RED_TT    292
#define IDS_MENU_BORDER_COLOR_GREEN     293
#define IDS_MENU_BORDER_COLOR_GREEN_TT  294
#define IDS_MENU_BORDER_COLOR_BLUE      295
#define IDS_MENU_BORDER_COLOR_BLUE_TT   296
#define IDS_MENU_BORDER_OPACITY         297
#define IDS_MENU_BORDER_OPACITY_TT      298
#define IDS_MENU_EXPORT_TO_FILE_SHAPE_TT 299
#define IDS_MENU_IMPORT_FROM_FILE_SHAPE_TT 300
#define IDS_ERROR_UNABLE_TO_SAVE_THE_FILE 303
#define IDS_SAVE_SUCCESSFUL             304
#define IDS_ERROR_UNABLE_TO_DELETE_THE_FILE 305
#define IDS_PRESET_X_DELETED            306
#define IDS_ERROR_A_FILE_ALREADY_EXISTS_WITH_THAT_FILENAME 307
#define IDS_ERROR_UNABLE_TO_RENAME_FILE 308
#define IDS_RENAME_SUCCESSFUL           309
#define IDS_SPRITE_X_WARNING_ERROR_IN_INIT_CODE 310
#define IDS_SPRITE_X_WARNING_ERROR_IN_PER_FRAME_CODE 311
#define IDS_SPRITE_X_ERROR_BAD_SLOT_INDEX 312
#define IDS_SPRITE_X_ERROR_IMAGE_FILE_MISSING_OR_CORRUPT 313
#define IDS_SPRITE_X_ERROR_OUT_OF_MEM   314
#define IDS_WARNING_PRESET_X_ERROR_IN_WAVE_X_INIT_CODE 319
#define IDS_WARNING_PRESET_X_ERROR_IN_WAVE_X_PER_FRAME_CODE 320
#define IDS_WARNING_PRESET_X_ERROR_IN_WAVE_X_PER_POINT_CODE 321
#define IDS_WARNING_PRESET_X_ERROR_IN_SHAPE_X_INIT_CODE 322
#define IDS_WARNING_PRESET_X_ERROR_IN_SHAPE_X_PER_FRAME_CODE 323
#define IDS_CONFIG_PANEL_BUTTON_1       324
#define IDS_CONFIG_PANEL_BUTTON_2       325
#define IDS_CONFIG_PANEL_BUTTON_3       326
#define IDS_CONFIG_PANEL_BUTTON_4       327
#define IDS_CONFIG_PANEL_BUTTON_5       328
#define IDS_CONFIG_PANEL_BUTTON_6       329
#define IDS_CONFIG_PANEL_BUTTON_7       330
#define IDS_CONFIG_PANEL_BUTTON_8       331
#define IDS_EXTRA_FONT_1_NAME           332
#define IDS_EXTRA_FONT_2_NAME           333
#define IDS_PRESS_F1_MSG                334
#define IDS_GRAPHICS_SUBSYSTEM_IS_TEMPORARILY_UNSTABLE 335
#define IDS_UNKNOWN                     336
#define IDS_DISABLED_PAGE_TEARING       337
#define IDS_NONE                        338
#define IDS_UNLIMITED                   339
#define IDS_X_FRAME_SEC                 340
#define IDS_HELP_ON_X_BUTTON            341
#define IDS_FONTS_HELP                  342
#define IDS_DUAL_HEAD_HELP              343
#define IDS_MULTI_SAMPLING              344
#define IDS_MULTI_SAMPLING_HELP         345
#define IDS_MAX_FRAMERATE               346
#define IDS_MAX_FRAMERATE_HELP          347
#define IDS_FAKE_FULLSCREEN             348
#define IDS_FAKE_FULLSCREEN_HELP        349
#define IDS_FULLSCREEN_ADAPTER          350
#define IDS_FULLSCREEN_ADAPTER_HELP     351
#define IDS_WINDOWED_ADPATER            352
#define IDS_WINDOWED_ADPATER_HELP       353
#define IDS_DESKTOP_ADAPTER             354
#define IDS_DESKTOP_ADAPTER_HELP        355
#define IDS_HELP_ON_X_CHECKBOX          356
#define IDS_HELP_ON_X_CHECKBOX_HELP     357
#define IDS_FORCE_INTO_FS_MODE_HELP     358
#define IDS_FORCE_INTO_DESKTOP_MODE_HELP 359
#define IDS_HELP_ON_F1                  360
#define IDS_HELP_ON_F1_HELP             361
#define IDS_CB_SKIN_HELP                362
#define IDS_SAVE_CPU_CHECKBOX           363
#define IDS_SAVE_CPU_CHECKBOX_HELP      364
#define IDS_FS_DISPLAY_MODE             365
#define IDS_FS_DISPLAY_MODE_HELP        366
#define IDS_TRY_TO_FIX_SLOW_TEXT        369
#define IDS_TRY_TO_FIX_SLOW_TEXT_HELP   370
#define IDS_VJ_MODE                     371
#define IDS_VJ_MODE_HELP                372
#define IDS_HELP_ON_X                   373
#define IDS_DMS_LABEL_HELP              374
#define IDS_FS_LABEL_HELP               375
#define IDS_W_LABEL_HELP                376
#define IDS_DM_MORE_HELP                377
#define IDS_INITCONFIG_FAILED           378
#define IDS_UNABLE_TO_LOAD_TABS         379
#define IDS_DOCUMENTATION_FILE_NOT_FOUND 380
#define IDS_ACCESS_TO_DOCUMENTATION_FILE_DENIED 381
#define IDS_ACCESS_TO_DOCUMENTATION_FILE_FAILED_DUE_TO_NO_ASSOC 382
#define IDS_ACCESS_TO_DOCUMENTATION_FILE_FAILED_CODE_X 383
#define IDS_ERROR_OPENING_DOCUMENTATION 384
#define IDS_URL_COULD_NOT_OPEN          385
#define IDS_ACCESS_TO_URL_WAS_DENIED    386
#define IDS_ACCESS_TO_URL_FAILED_DUE_TO_NO_ASSOC 387
#define IDS_ACCESS_TO_URL_FAILED_CODE_X 388
#define IDS_ERROR_OPENING_URL           389
#define IDS_RESTORE_ALL_DEFAULTS        390
#define IDS_RESTORE_ALL_DEFAULTS_TITLE  391
#define IDS_OK_HELP                     392
#define IDS_CANCEL_HELP                 393
#define IDS_RESTORE_DEFAULTS_HELP       394
#define IDS_DOCUMENTATION_BUTTON_HELP   395
#define IDS_VIEW_ONLINE_DOCS_HELP       396
#define IDS_5_6_5_TEXTURE               397
#define IDS_5_5_5_TEXTURE               398
#define IDS_8_8_8_TEXTURE               399
#define IDS_NO_ALPHA_FALLBACK           400
#define IDS_NO_ALPHA_FALLBACK_HELP      401
#define IDS_CB_SHOW_ICONS_HELP          402
#define IDS_CB_BOX                      403
#define IDS_CB_BOX_HELP                 404
#define IDS_CB_MANUAL_SCOOT             405
#define IDS_CB_MANUAL_SCOOT_HELP        406
#define IDS_SPAN_BOTH_SCREENS           407
#define IDS_USE_LEFT_SCREEN_ONLY        408
#define IDS_USE_RIGHT_SCREEN_ONLY       409
#define IDS_USE_TOP_SCREEN_ONLY         410
#define IDS_USE_BOTTOM_SCREEN_ONLY      411
#define IDS_COULD_NOT_FIND_FILE_FOR_DESKTOP_MODE_X 412
#define IDS_MILKDROP_ERROR_FILE_MISSING 413
#define IDS_ERROR_CREATING_GDI_DESKTOP_FONT 414
#define IDS_ERROR_CREATING_DESKTOP_FONT 415
#define IDS_ERROR_CREATING_TEXTURE_FOR_ICON_BITMAPS 416
#define IDS_OUTDATED_VMS_DESKTOP_DLL_NEED_TO_REINSTALL 417
#define IDS_ERROR_CREATING_HOOK_PROC_DESKTOP_ICONS_NOT_AVAILABLE 418
#define IDS_ERROR_UPDATING_ICON_BITMAPS 419
#define IDS_ERROR_UPDATING_ICON_BITMAPS_TOO_MANY_UNIQUE_ICON_BITMAPS 420
#define IDS_ERROR_UPDATING_ICON_BITMAPS_COULD_NOT_GET_LEVEL_DESCRIPTION 421
#define IDS_ERROR_UPDATING_ICON_BITMAPS_LOCKRECT_FAILED 422
#define IDS_ERROR_UPDATING_ICON_BITMAPS_LR_PBITS_IS_NULL 423
#define IDS_ERROR_UPDATING_ICON_BITMAPS_UNKNOWN_PIXEL_FORMAT 424
#define IDS_ERROR_UPDATING_ICON_BITMAPS_COULDNT_GET_HDC 425
#define IDS_ERROR_UPDATING_ICON_BITMAPS_CALL_TO_GETDIBITS_FAILED 426
#define IDS_ERROR_UPDATING_ICON_BITMAPS_CALL_2_TO_GETDIBITS_FAILED 427
#define IDS_ERROR_UPDATING_ICON_BITMAPS_GETICONINFO_FAILED 428
#define IDS_ERROR_UPDATING_ICON_BITMAPS_SHGETFILEINFO_FAILED 429
#define IDS_UNABLE_TO_REGISTER_WINDOW_CLASS 430
#define IDS_DIRECTX_INIT_FAILED         431
#define IDS_DXC_ERR_CAPSFAIL            432
#define IDS_FS_DISPLAY_MODE_SELECTED_IS_INVALID 433
#define IDS_CREATEWINDOW_FAILED         434
#define IDS_TRYING_TO_ENTER_FS_MODE_WITH_MULTIPLE_DISPLAYS 435
#define IDS_TIP                         436
#define IDS_TRYING_TO_ENTER_FS_MODE_WITH_MULTIPLE_DISPLAYS_2 437
#define IDS_UNABLE_TO_CREATE_DIRECTX_DEVICE 438
#define IDS_OLDER_DISPLAY_ADAPTER_CATENATION 439
#define IDS_OLDER_DISPLAY_ADAPTER_CATENATION_2 440
#define IDS_DIRECTX_INIT_FAILED_X       441
#define IDS_WINDOW_RESIZE_FAILED        442
#define IDS_OUT_OF_VIDEO_MEMORY         443
#define IDS_ERROR_CREATING_GDI_FONTS    444
#define IDS_ERROR_LOADING_MAIN_MENU     445
#define IDS_ERROR_LOADING_CONTEXT_MENU  446
#define IDS_ERROR_CREATING_DIRECT3D_DEVICE_FOR_VJ_MODE 447
#define IDS_VJ_MODE_INIT_ERROR          448
#define IDS_ERROR_REGISTERING_WINDOW_CLASS_FOR_TEXT_WINDOW 449
#define IDS_ERROR_CREATING_VJ_WINDOW    450
#define IDS_ERROR_CREATING_D3D_DEVICE_FOR_VJ_MODE 451
#define IDS_ERROR_CREATING_D3DX_FONTS   452
#define IDS_UNABLE_TO_INIT_DXCONTEXT    453
#define IDS_TO_FREE_UP_SOME_MEMORY_RESTART_WINAMP_THEN_GO_TO_CONFIG 454
#define IDS_TO_FREE_UP_SOME_MEMORY_RESTART_WINAMP_THEN_GO_TO_CONFIG_2 455
#define IDS_TO_FREE_UP_SOME_MEMORY_RESTART_WINAMP_THEN_GO_TO_CONFIG_3 456
#define IDS_TO_FREE_UP_VIDEO_MEMORY     457
#define IDS_TO_FREE_UP_VIDEO_MEMORY_2   458
#define IDS_MILKDROP_SUGGESTION         459
#define IDS_DIRECTX_MISSING_OR_CORRUPT  460
#define IDS_ERROR_THE_PLUGIN_IS_ALREADY_RUNNING 463
#define IDS_THIS_PLUGIN_NEEDS_MUSIC_TO_RUN 464
#define IDS_NO_MUSIC_PLAYING            465
#define IDS_UNABLE_TO_READ_DATA_FILE_X  466
#define IDS_COULD_NOT_CREATE_MY_VERTEX_DECLARATION 467
#define IDS_COULD_NOT_CREATE_WF_VERTEX_DECLARATION 468
#define IDS_COULD_NOT_CREATE_SPRITE_VERTEX_DECLARATION 469
#define IDS_SHADER_MODEL_2              470
#define IDS_SHADER_MODEL_3              471
#define IDS_SHADER_MODEL_4              472
#define IDS_UKNOWN_CASE_X               473
#define IDS_FAILED_TO_COMPILE_PIXEL_SHADERS_USING_X 474
#define IDS_FAILED_TO_COMPILE_PIXEL_SHADERS_HARDWARE_MIS_REPORT 475
#define IDS_COULD_NOT_COMPILE_FALLBACK_WV_SHADER 476
#define IDS_COULD_NOT_COMPILE_FALLBACK_CV_SHADER 477
#define IDS_COULD_NOT_COMPILE_FALLBACK_CP_SHADER 478
#define IDS_COULD_NOT_COMPILE_BLUR1_VERTEX_SHADER 479
#define IDS_COULD_NOT_COMPILE_BLUR1_PIXEL_SHADER 480
#define IDS_COULD_NOT_COMPILE_BLUR2_VERTEX_SHADER 481
#define IDS_COULD_NOT_COMPILE_BLUR2_PIXEL_SHADER 482
#define IDS_COULD_NOT_CREATE_INTERNAL_CANVAS_TEXTURE_SMALLER_DISPLAY 483
#define IDS_COULD_NOT_CREATE_INTERNAL_CANVAS_TEXTURE_NOT_ENOUGH_VID_MEM_RECOMMENDATION 484
#define IDS_COULD_NOT_CREATE_INTERNAL_CANVAS_TEXTURE_NOT_ENOUGH_VID_MEM 485
#define IDS_SUCCESSFULLY_CREATED_VS0_VS1 486
#define IDS_ERROR_CREATING_BLUR_TEXTURES 487
#define IDS_COULD_NOT_CREATE_NOISE_TEXTURE 488
#define IDS_COULD_NOT_LOCK_NOISE_TEXTURE 489
#define IDS_NOISE_TEXTURE_BYTE_LAYOUT_NOT_RECOGNISED 490
#define IDS_COULD_NOT_CREATE_3D_NOISE_TEXTURE 491
#define IDS_COULD_NOT_LOCK_3D_NOISE_TEXTURE 492
#define IDS_3D_NOISE_TEXTURE_BYTE_LAYOUT_NOT_RECOGNISED 493
#define IDS_ERROR_CREATING_SHADER       498
#define IDS_PLEASE_EXIT_VIS_BEFORE_RUNNING_CONFIG_PANEL 499
#define IDS_FPS                         500
#define IDS_PF_MONITOR                  501
#define IDS_PRESS_F9_TO_HIDE_SHADER_QREF 502
#define IDS_PRESS_F9_TO_SHOW_SHADER_QREF 503
#define IDS_WARP_AND_COMPOSITE_SHADERS_LOCKED 504
#define IDS_WARP_SHADER_LOCKED          505
#define IDS_COMPOSITE_SHADER_LOCKED     506
#define IDS_PRESET_USES_HIGHEST_PIXEL_SHADER_VERSION 507
#define IDS_PRESET_HAS_MIXED_VERSIONS_OF_SHADERS 508
#define IDS_UPGRADE_SHADERS_TO_USE_PS2  509
#define IDS_UPGRADE_SHADERS_TO_USE_PS3  510
#define IDS_PRESET_DOES_NOT_USE_PIXEL_SHADERS 511
#define IDS_UPGRADE_TO_USE_PS2          512
#define IDS_WARNING_OLD_GPU_MIGHT_NOT_WORK_WITH_PRESET 513
#define IDS_PRESET_CURRENTLY_USES_PS2   514
#define IDS_UPGRADE_TO_USE_PS3          515
#define IDS_PRESET_CURRENTLY_USES_PS3   516
#define IDS_UPGRADE_TO_USE_PS4          517
#define IDS_WARNING_DO_NOT_FORGET_WARP_SHADER_WAS_LOCKED 518
#define IDS_WARNING_DO_NOT_FORGET_COMPOSITE_SHADER_WAS_LOCKED 519
#define IDS_PRESET_MASH_UP_TEXT1        520
#define IDS_PRESET_CURRENTLY_USES_PS2X  521
#define IDS_UPGRADE_TO_USE_PS2X         522
#define IDS_PRESET_MASH_UP_TEXT2        524
#define IDS_PRESET_MASH_UP_TEXT3        525
#define IDS_PRESET_MASH_UP_TEXT4        526
#define IDS_DIRECTORY_OF_X              527
#define IDS_SHUFFLE_IS_NOW_OFF          528
#define IDS_SHUFFLE_IS_NOW_ON           529
#define IDS_COMPSHADER_LOCKED           530
#define IDS_WARPSHADER_LOCKED           531
#define IDS_ALLSHADERS_LOCKED           532
#define IDS_ALLSHADERS_UNLOCKED         533
#define IDS_UPGRADE_TO_USE_PS2B         534
#define IDS_PS_AUTO_RECOMMENDED         535
#define IDS_PS_DISABLED                 536
#define IDS_PS_SHADER_MODEL_2           537
#define IDS_PS_SHADER_MODEL_3           538
#define IDS_TX_8_BITS_PER_CHANNEL       539
#define IDS_TX_16_BITS_PER_CHANNEL      540
#define IDS_TX_32_BITS_PER_CHANNEL      541
#define IDS_160X120_SLOW                542
#define IDS_192X144_SLOW                543
#define IDS_NONE_BEST_IMAGE_QUALITY     544
#define IDS_1_25_X                      545
#define IDS_1_33_X                      546
#define IDS_1_5_X                       547
#define IDS_1_67_X                      548
#define IDS_2_X                         549
#define IDS_3_X                         550
#define IDS_4_X                         551
#define IDS_NEAREST_POWER_OF_2          552
#define IDS_EXACT_RECOMMENDED           553
#define IDS_PIXEL_SHADERS               554
#define IDS_PIXEL_SHADERS_TEXT          555
#define IDS_TEXFORMAT                   556
#define IDS_TEXFORMAT_TEXT              557
#define IDS_CANVAS_SIZE                 558
#define IDS_CANVAS_SIZE_TEXT            559
#define IDS_CANVAS_STRETCH              560
#define IDS_CANVAS_STRETCH_TEXT         561
#define IDS_MAX_IMAGES_BYTES_TEXT       562
#define IDS_MENU_EDIT_WARP_SHADER       563
#define IDS_MENU_EDIT_COMPOSITE_SHADER  564
#define IDS_MENU_BLUR1_EDGE_DARKEN_AMOUNT 565
#define IDS_MENU_BLUR1_EDGE_DARKEN_AMOUNT_TT 566
#define IDS_MENU_BLUR1_MIN_COLOR_VALUE  567
#define IDS_MENU_BLUR1_MIN_COLOR_VALUE_TT 568
#define IDS_MENU_BLUR1_MAX_COLOR_VALUE  569
#define IDS_MENU_BLUR1_MAX_COLOR_VALUE_TT 570
#define IDS_MENU_BLUR2_MIN_COLOR_VALUE  571
#define IDS_MENU_BLUR2_MIN_COLOR_VALUE_TT 572
#define IDS_MENU_BLUR2_MAX_COLOR_VALUE  573
#define IDS_MENU_BLUR2_MAX_COLOR_VALUE_TT 574
#define IDS_MENU_BLUR3_MIN_COLOR_VALUE  575
#define IDS_MENU_BLUR3_MIN_COLOR_VALUE_TT 576
#define IDS_MENU_BLUR3_MAX_COLOR_VALUE  577
#define IDS_MENU_BLUR3_MAX_COLOR_VALUE_TT 578
#define IDS_MENU_EDIT_PER_VERTEX_EQUATIONS 579
#define IDS_MENU_EDIT_PER_VERTEX_EQUATIONS_TT 580
#define IDS_MENU_EDIT_WARP_SHADER_TT    581
#define IDS_MENU_EDIT_COMPOSITE_SHADER_TT 582
#define IDS_MENU_EDIT_UPGRADE_PRESET_PS_VERSION 583
#define IDS_MENU_EDIT_UPGRADE_PRESET_PS_VERSION_TT 584
#define IDS_MENU_EDIT_DO_A_PRESET_MASH_UP 585
#define IDS_MENU_EDIT_DO_A_PRESET_MASH_UP_TT 586
#define IDS_MENU_NUMBER_OF_INSTANCES    587
#define IDS_MENU_NUMBER_OF_INSTANCES_TT 588
#define IDS_MENU_EDIT_INIT_CODE_SHAPE_TT 589
#define IDS_MENU_EDIT_PER_FRAME_INSTANCE_CODE 590
#define IDS_MENU_EDIT_PER_FRAME_INSTANCE_CODE_TT 591
#define IDS_ERROR_PRESET_NOT_FOUND_X    592
#define IDS_ERROR_NO_PRESET_FILES_OR_DIRS_FOUND_IN_X 593
#define IDS_SCANNING_PRESETS            594
#define IDS_SPRITE_X_ERROR_COULD_NOT_FIND_IMG_OR_NOT_DEFINED 595
#define IDS_WARNING_PRESET_X_ERROR_IN_PRESET_INIT_CODE 596
#define IDS_WARNING_PRESET_X_ERROR_IN_PER_FRAME_CODE 597
#define IDS_WARNING_PRESET_X_ERROR_IN_PER_VERTEX_CODE 598
#define IDS_HZ                          599
#define IDS_HELP_MINIMIZE_WINAMP        600
#define IDS_HELP_MINIMIZE_WINAMP_HELP   601
#define IDS_DIRECTX_MISSING_OR_CORRUPT_TEXT 602
#define IDS_PARENT_DIRECTORY            603
#define IDS_RAND_TITLE                  604
#define IDS_RAND_MSG                    605
#define IDS_MAX_IMAGES_BYTES            606
#define IDS_PAGE_X                      607
#define IDS_MB                          608
#define IDS_GB                          609
#define IDS_MASHUP_GENERAL_POSTPROC     610
#define IDS_MASHUP_MOTION_EQUATIONS     611
#define IDS_MASHUP_WAVEFORMS_SHAPES     612
#define IDS_MASHUP_WARP_SHADER          613
#define IDS_MASHUP_COMP_SHADER          614
#define IDS_STRING615                   615
#define IDS_STRING616                   616
#define IDS_STRING617                   617
#define IDS_STRNG618                    618
#define IDS_STRING618                   618
#define IDS_STRING619                   619
#define IDS_STRING620                   620
#define IDS_STRING621                   621
#define IDS_STRING622                   622
#define IDS_STRING623                   623
#define IDS_STRING624                   624
#define IDS_STRING625                   625
#define IDS_STRING626                   626
#define IDS_STRING627                   627
#define IDS_STRING628                   628
#define IDS_STRING629                   629
#define IDS_STRING630                   630
#define IDS_STRING631                   631
#define IDS_STRING632                   632
#define IDS_STRING633                   633
#define IDS_STRING634                   634
#define IDS_STRING635                   635
#define IDS_UPGRADE_SHADERS_TO_USE_PS2X 636
#define IDS_PRESS_ESC_TO_RETURN         637
#define IDS_COULD_NOT_LOAD_TEXTURE_X    638
#define IDS_ERROR_COMPILING_X_X_SHADER  639
#define IDS_ERROR_PARSING_X_X_SHADER    640
#define IDS_UNABLE_TO_RESOLVE_TEXSIZE_FOR_A_TEXTURE_NOT_IN_USE 641
#define IDS_KEY_MAPPINGS                642
#define IDC_CB_FOG                      1000
#define IDC_CB_SUPERTEX                 1001
#define IDC_CB_HELP_MSG                 1001
#define IDC_CB_PRESS_F1_MSG             1001
#define IDC_DISP_MODE                   1003
#define IDC_CB_FS                       1004
#define IDC_CB_PT                       1005
#define IDC_CB_WPT                      1005
#define IDC_SZ_ABOUT                    1006
#define IDC_CB_MIN                      1006
#define IDC_ADAPTER_W                   1007
#define IDC_ADAPTER_TEXT                1008
#define IDC_ADAPTER_W_CAPTION           1008
#define IDC_ADAPTER_FFS                 1008
#define IDC_ADAPTER_DMS                 1008
#define IDC_DISP_MODE_TEXT              1009
#define IDC_DISP_MODE_CAPTION           1009
#define IDC_ADAPTER_FS                  1010
#define IDC_ADAPTER_TEXT2               1011
#define IDC_ADAPTER_FS_CAPTION          1011
#define IDC_FS_ADAPTER_CAPTION          1011
#define IDC_CB_TWOPASS                  1012
#define IDC_CB_FFSPT                    1012
#define IDC_CB_DMSPT                    1012
#define IDC_CB_TRANS_HAIR               1013
#define IDC_ADAPTER_FS_CAPTION2         1013
#define IDC_W_ADAPTER_CAPTION           1013
#define IDC_FONT2b                      1013
#define IDC_FONT2                       1013
#define IDC_CB_3B                       1014
#define IDC_ADD_STUFF_HERE              1014
#define IDC_FFS_ADAPTER_CAPTION         1014
#define IDC_DMS_ADAPTER_CAPTION         1014
#define IDC_CB_DEBUGOUTPUT              1014
#define IDC_FONT3b                      1014
#define IDC_FONT3                       1014
#define IDC_MQ                          1015
#define IDC_CB_FAKE                     1015
#define IDC_TEXSIZECOMBO                1015
#define IDC_FONT4b                      1015
#define IDC_FONT4                       1015
#define IDC_DISP_MODE_TEXT2             1016
#define IDC_CB_NO_FAKE_TIPS             1016
#define IDC_TAB1                        1016
#define IDC_TABS                        1016
#define IDC_CB_FSPT                     1016
#define IDC_FONTSIZE                    1016
#define IDC_CB_AUTOGAMMA                1016
#define IDC_FONT5b                      1016
#define IDC_FONT5                       1016
#define IDC_WMS                         1017
#define IDC_MESHSIZECOMBO               1017
#define IDC_FONTSIZE3                   1017
#define IDC_DISP_MODE_TEXT3             1018
#define IDC_MULTISAMPLING_CAPTION2      1018
#define IDC_FS_BOX                      1018
#define IDC_W_MULTISAMPLING_CAPTION     1018
#define IDC_BETWEEN_TIME                1018
#define IDC_FONTSIZE4                   1018
#define IDC_FSMS                        1019
#define IDC_CB_ALWAYS3D                 1019
#define IDC_BETWEEN_TIME_RANDOM         1019
#define IDC_FONT6b                      1019
#define IDC_FONT6                       1019
#define IDC_DISP_MODE_TEXT4             1020
#define IDC_MULTISAMPLING_CAPTION       1020
#define IDC_FRACTAL_RES_CAPTION         1020
#define IDC_INITIAL_WAVEMODE_CAPTION    1020
#define IDC_CB_DMS                      1020
#define IDC_HARDCUT_BETWEEN_TIME        1020
#define IDC_FONTSIZE5                   1020
#define IDC_CB_SCROLLON                 1020
#define IDC_FS_LIMIT                    1021
#define IDC_FS_MAXFPS                   1021
#define IDC_FONTOPTIONS                 1021
#define IDC_TEXFORMAT                   1021
#define IDC_DISP_MODE_TEXT5             1022
#define IDC_FRAME_DELAY_CAPTION         1022
#define IDC_FS_FRAME_DELAY_CAPTION      1022
#define IDC_FS_MAXFPS_CAPTION           1022
#define IDC_FONTOPTIONS3                1022
#define IDC_SHADERS                     1022
#define IDC_W_LIMIT                     1023
#define IDC_W_MAXFPS                    1023
#define IDC_CB_NOWARN                   1023
#define IDC_BLEND_AUTO                  1023
#define IDC_CB_SCROLLON2                1023
#define IDC_DISP_MODE_TEXT6             1024
#define IDC_FRAME_DELAY_CAPTION2        1024
#define IDC_G_BOX                       1024
#define IDC_W_FRAME_DELAY_CAPTION       1024
#define IDC_W_MAXFPS_CAPTION            1024
#define IDC_CB_NOWARN2                  1024
#define IDC_BLEND_USER                  1024
#define IDC_W_BOX                       1025
#define IDC_CB_SAVE_CPU                 1025
#define IDC_FONT7b                      1025
#define IDC_FONT7                       1025
#define IDC_CB_ANISO                    1025
#define IDC_TABS_PLACEHOLDER            1026
#define IDC_FFSMS                       1026
#define IDC_DMSMS                       1026
#define IDC_CB_NORATING                 1026
#define IDC_MULTISAMPLING_CAPTION3      1027
#define IDC_FFS_MULTISAMPLING_CAPTION   1027
#define IDC_FONTSIZE2                   1027
#define IDC_DMS_MULTISAMPLING_CAPTION   1027
#define IDC_CB_INSTASCAN                1027
#define IDC_FRAME_DELAY_CAPTION3        1028
#define IDC_FFS_FRAME_DELAY_CAPTION     1028
#define IDC_FFS_MAXFPS_CAPTION          1028
#define IDC_FONTOPTIONS2                1028
#define IDC_DMS_MAXFPS_CAPTION          1028
#define IDC_STRETCH                     1028
#define IDC_FFS_LIMIT                   1029
#define IDC_FFS_MAXFPS                  1029
#define IDC_DMS_MAXFPS                  1029
#define IDC_FONTSIZE6                   1029
#define IDC_MAX_IMAGES                  1029
#define IDC_RECT                        1030
#define IDC_CB_SKIN                     1030
#define IDC_MAX_BYTES                   1030
#define IDC_Q                           1031
#define IDC_FRACTAL_RES                 1031
#define IDC_INITIAL_WAVEMODE            1031
#define IDC_FONT8b                      1031
#define IDC_FONT8                       1031
#define IDC_CB_MIN2                     1031
#define IDC_CB_FIXSLOWTEXT              1031
#define IDC_HS_ANIM_SPEED               1032
#define IDC_BETWEEN_TIME_LABEL          1032
#define IDC_FONTSIZE7                   1032
#define IDC_FS_MULTISAMPLING_CAPTION    1033
#define IDC_BETWEEN_TIME_RANDOM_LABEL   1033
#define IDC_HS_ANIM_SPEED_CAPTION       1034
#define IDC_BLEND_AUTO_LABEL            1034
#define IDC_FONT9b                      1034
#define IDC_FONT9                       1034
#define IDC_BLEND_USER_LABEL            1035
#define IDC_FONTSIZE8                   1035
#define IDC_FFS_LABEL                   1036
#define IDC_DMS_LABEL                   1036
#define IDC_BLEND_USER_LABEL2           1036
#define IDC_BETWEEN_TIME_LABEL2         1036
#define IDC_RAND_TITLE_LABEL            1036
#define IDC_HARDCUT_BETWEEN_TIME_LABEL  1036
#define IDC_FS_LABEL                    1037
#define IDC_BLEND_USER_LABEL3           1037
#define IDC_RAND_MSG_LABEL              1037
#define IDC_W_LABEL                     1038
#define IDC_FONTSIZE9                   1038
#define IDC_3DSEP_LABEL                 1038
#define ID_FONTS                        1039
#define IDC_BRIGHT_SLIDER               1039
#define IDC_FONT_CAPTION                1040
#define ID_DM_MORE                      1040
#define IDC_FONTSIZE_CAPTION            1041
#define ID_DUALHEAD                     1041
#define IDC_FONTOPTIONS_CAPTION         1042
#define IDC_SYSTEM_FONT                 1043
#define IDC_CB_SEPTEXT                  1043
#define IDC_FONT_NAME_2                 1043
#define IDC_DECORATIVE_FONT             1044
#define IDC_HARDCUT_LOUDNESS            1044
#define IDC_FONT_NAME_1                 1044
#define IDC_CB_HARDCUTS                 1045
#define IDC_TITLE_FONT                  1045
#define IDC_FONT_NAME_3                 1045
#define IDC_CB_BOX                      1046
#define IDC_SONGTITLEANIM_DURATION      1046
#define IDC_FONT_NAME_4                 1046
#define IDC_DM_ALPHA_FIX                1047
#define IDC_CB_TITLE_ANIMS              1047
#define IDC_FONT_NAME_5                 1047
#define IDC_DM_ALPHA_FIX_CAPTION        1048
#define IDC_V_PICK                      1048
#define IDC_RAND_TITLE                  1048
#define IDC_FONT_NAME_6                 1048
#define IDC_CB_MANUAL_SCOOT             1049
#define IDC_T1                          1049
#define IDC_RAND_MSG                    1049
#define IDC_FONT_NAME_7                 1049
#define IDC_T3                          1050
#define IDC_CB_SHOW_ICONS               1050
#define IDC_FONT_NAME_8                 1050
#define IDC_3DSEP                       1050
#define IDC_H_PICK                      1051
#define IDC_T5                          1051
#define IDC_FONT_NAME_9                 1051
#define IDC_T2                          1052
#define IDC_FONTOPTIONS_CAPTION2        1052
#define IDC_T4                          1053
#define IDC_FONTOPTIONS_CAPTION3        1053
#define IDC_FONT_TEXT                   1054
#define IDC_TEXSIZECOMBO_CAPTION        1054
#define IDC_MESHSIZECOMBO_CAPTION       1055
#define IDC_BRIGHT_SLIDER_BOX           1056
#define IDC_SONGTITLEANIM_DURATION_LABEL 1057
#define IDC_TEXFORMAT_CAPTION           1057
#define IDC_HARDCUT_LOUDNESS_LABEL      1058
#define IDC_SHADERS_CAPTION             1058
#define IDC_FONTSIZE1                   1059
#define IDC_HARDCUT_LOUDNESS_MIN        1059
#define IDC_BRIGHT_SLIDER_BOX2          1059
#define IDC_HARDCUT_LOUDNESS_MAX        1060
#define IDC_STRETCH_CAPTION             1060
#define IDC_CB_VJMODE                   1061
#define IDC_MAX_IMAGES_CAPTION          1061
#define IDC_MAX_BYTES_CAPTION           1062
#define IDC_FONTBOLD1                   1063
#define IDC_FONTITAL1                   1064
#define IDC_FONTAA1                     1065
#define IDC_FONTBOLD2                   1066
#define IDC_FONTITAL2                   1067
#define IDC_FONTAA2                     1068
#define IDC_FONTBOLD3                   1069
#define IDC_FONTITAL3                   1070
#define IDC_FONTAA3                     1071
#define IDC_FONTBOLD4                   1072
#define IDC_FONTITAL4                   1073
#define IDC_FONTAA4                     1074
#define IDC_FONTBOLD5                   1075
#define IDC_FONTITAL5                   1076
#define IDC_FONTAA5                     1077
#define IDC_FONTBOLD6                   1078
#define IDC_FONTITAL6                   1079
#define IDC_FONTAA6                     1080
#define IDC_FONTBOLD7                   1084
#define IDC_FONTITAL7                   1085
#define IDC_FONTAA7                     1086
#define IDC_FONTBOLD8                   1087
#define IDC_FONTITAL8                   1088
#define IDC_FONTAA8                     1089
#define IDC_FONTBOLD9                   1090
#define IDC_FONTITAL9                   1091
#define IDC_FONTAA9                     1092
#define IDC_FONT1b                      1093
#define IDC_FONT1                       1093
#define ID_QUIT                         40006
#define ID_GO_FS                        40007
#define ID_SHOWHELP                     40008
#define ID_SHOWPLAYLIST                 40009
#define ID_DESKTOP_MODE                 40010

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        131
#define _APS_NEXT_COMMAND_VALUE         40011
#define _APS_NEXT_CONTROL_VALUE         1062
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
