/* (Auto-generated binary data file). */

#ifndef BINARY_DISTRHOARTWORKCYCLESHIFTER_HPP
#define BINARY_DISTRHOARTWORKCYCLESHIFTER_HPP

namespace DistrhoArtworkCycleShifter
{
    extern const char* backData;
    const unsigned int backDataSize = 57352;
    const unsigned int backWidth    = 268;
    const unsigned int backHeight   = 107;

    extern const char* sliderData;
    const unsigned int sliderDataSize = 392;
    const unsigned int sliderWidth    = 14;
    const unsigned int sliderHeight   = 14;
}

#endif // BINARY_DISTRHOARTWORKCYCLESHIFTER_HPP

